[<p align="right">->Back to directory</p>](0.directory.md)


# owlcache Configuration file
* When the program starts, the parameters you dynamically configure will overwrite the parameters in the *.conf file

## Profile writing convention
>1.Indicates that this line is a comment at the beginning of #    
>2.Grouping information with [] (more intuitive display of configuration information)   
>3.The configuration parameter name and parameter value are separated by a space.     

## [Owlcache]
### Set the IP that the program listens
* If you cannot access the service after normal startup, please check whether the firewall or other software restricts your access.
>Host 0.0.0.0

### Set program response IP
* Default local
* Use it in the response result Json string returned by TCP and HTTP, pay attention to the correctness of the IP during actual deployment in the local area network and external network environment!
>ResponseHost 127.0.0.1

### Log file path
* The default log will be generated to the service running directory
* !!! If the Logfile value is specified when starting the service on the command line, and the configuration file is not commented out here, the configuration file shall prevail
>Logfile ./log_file/

### Database file path
* The default database file will be generated to the service running directory
>DBfile ./db_file/


## [TCP]
### TCP listening port
* The default is 7720
* If it is not filled in or the port value is 0, port 7720 will be monitored by default
>Tcpport 7720

### Whether to open the Tcp service
* No password authentication in TCP mode (for production environment deployment, please shield TCP access ports according to the actual situation, etc., pay attention to use safety)
* The values are "1" (open service) and "0" (close service). The default is 1 to enable the service.
>CloseTcp 1


## [HTTP]
### HTTP listening port
* The default is 7721.
* If it is not filled in or the port value is 0, port 7721 will be monitored by default.
>Httpport 7721

### HTTP password
* Mandatory and necessary, don’t set too simple
* Please set your password. The length of the password must not be less than ten digits, and it cannot be a pure number. It is recommended to use uppercase letters + lowercase letters + numbers + simple symbols
>Pass 

### Tonken expiration time
* Set an expiration time for the Tonken value generated by the "Pass" command.
* The default is 0. The unit is seconds. If the value is 0, it will never expire.
* Warning: The setting never expires, there may be a security risk.
>Tonken_expire_time 0

### HTTPS related configuration
* Open_Https Whether to open HTTPS. Values are 0 (closed), 1 (open). Off by default.
* Https_CertFile Cert file path. For example: /www/server.crt, D:/key/server.crt.
* Https_KeyFile Key file path. For example: /www/server.key, D:/key/server.key.
* HttpsClient_InsecureSkipVerify When the HTTPS mode is enabled, whether to verify the validity of the certificate when owlcache communicates with each other. Values are 0 (closed), 1 (open). Off by default. The validity of the certificate will not be verified when it is turned on.
>Open_Https 0

>Https_CertFile /www/server.crt

>Https_KeyFile /www/server.key

>HttpsClient_InsecureSkipVerify 0

### Cross-domain related configuration
* "Cross-origin resource sharing" configuration.
* Cors values are "1" (open service) and "0" (close service). The default value is 0 to close the service and does not allow cross-domain.
* The Access_Control_Allow_Origin value defaults to "*" (allow all domain requests). You can set this value to the domain you specify.
* Note: Be cautious of CSRF attacks.
>Cors 0

>Access_Control_Allow_Origin *


## [HttpGroupData]
### Request timeout time when the clusters communicate with each other
* The unit is milliseconds. The default timeout is 2700 milliseconds. In fact, the data intercommunication time of the intranet environment is very low, and it can be set according to the actual situation.
>HttpClientRequestTimeout 2700

### Http client request settings
* The requested data will be cached locally for a short time to avoid repeated frantically querying other nodes for the same Key in a high concurrency state. Appropriate delayed query helps optimize performance and efficiency.
* The unit is milliseconds, and the default lifetime of this cache is 5000 milliseconds. A value of 0 means no caching (suitable for scenarios with small concurrency and high data real-time requirements).
>HttpClientRequestLocalCacheLifeTime 5000


## [GroupDataSync]

### Whether to enable cluster data synchronization
* 0 means not open. 1 means on. It is not turned on by default.
>GroupDataSync 0

### Set Gossip listening port
* This item will take effect only when the cluster data synchronization service is enabled.
* The cluster data synchronization service relies on the Gossip protocol. It is recommended not to set this item, the system will automatically allocate ports. The default value is 0 (the system automatically monitors a port and outputs the port at startup).
>Gossipport 0

### The entire Gossip cluster needs a unified password
* This item will take effect only when the cluster data synchronization service is enabled.
* A token used to exchange data in the cluster through the Gossip protocol. The default is blank, you need to set the same password for the cluster.
>GossipDataSyncAuthKey 

### Request timeout time when cluster synchronization data communicate with each other
* The unit is milliseconds. The default timeout is 5000 milliseconds. In fact, the data intercommunication time of the intranet environment is very low, and it can be set according to the actual situation.
>GossipHttpClientRequestTimeout 5000


## [Memcache_Client]
### Turn on importing data from memcache
* Whether to enable query for non-existent data, query from memcache and save it in the local database. 0 means not open.
>Get_data_from_memcache 0

### List of memcache queried
* The "|" symbol interval is required between different nodes. For example: 127.0.0.1:11211|192.168.0.77:11211. The default value is empty.
>Memcache_list 

### Expiration time of imported data
* Set an expiration time for the key stored in the local database from memcache. The default is 0, which will never expire. The unit is seconds.
>Get_memcache_data_set_expire_time 0

### MemcacheClient client request settings
* When Max_Error_Number exceeds the limit value, MemcacheClient requests to "pause" the Sleeptime value to optimize the response speed of the program.
* Sleeptime represents sleep time, in seconds, the default minimum sleep is 2 seconds.
* Max_Error_Number represents the maximum number of failed requests, the unit is an integer, and the default minimum error request is 2 times.
>MemcacheClient_Request_Timeout_Sleeptime 2

>MemcacheClient_Request_Max_Error_Number 2


## [Redis_Client]
### Turn on importing data from Redis
* Whether to open the query for non-existent data, query from redis and save it in the local database. 0 means not open.
>Get_data_from_redis 0

### The Redis address to be queried
* Cannot be a null value.  
>Redis_Addr 

### Redis password to be queried
* Cannot be a null value.  
>Redis_Password 

### Need to query Redis database
* Default is 0
>Redis_DB 0

### Expiration time of imported data
* Set an expiration time for the key stored in the local database from Redis. The default is 0, which will never expire. The unit is seconds.
>Get_redis_data_set_expire_time 0

### RedisClient client requests settings
* The RedisClient client requests settings. When Max_Error_Number exceeds the limit value, RedisClient requests to "pause" the Sleeptime value to optimize the response speed of the program.
* Sleeptime represents sleep time, in seconds, the default minimum sleep is 2 seconds.
* Max_Error_Number represents the maximum number of failed requests, the unit is an integer, and the default minimum error request is 2 times.
>RedisClient_Request_Timeout_Sleeptime 2

>RedisClient_Request_Max_Error_Number 2


## [Url_Cache]
### Enable importing data from URL address
* Cache the HTTP response data obtained after URL proxy access to Owlcache
* The values are "1" (open service) and "0" (close service). The default is 0 to turn off the service.
>Open_Urlcache 0

### The name of the configuration file that needs to be loaded after URL caching is enabled
* The format is XML, it exists in the DBfile configuration item directory, and the default name is sites.xml.
>Urlcache_Filename sites.xml

### Enable quick access to Urlcache
* Does not affect the default access method of UrlCache.
* Values are "1" (on) and "0" (off). The default is 0 to close.
>Urlcache_Request_Easy 0


## [Task]
### Timeer configuration--storage time for automatic backup of DB data
* The unit is in minutes. The default is 1.
* Warning: If there is a lot of data in the database, when the service is restarted, if the time to reload the data into the memory exceeds the set backup time, the data entity file will be overwritten, which is a very serious data loss error.  
>Task_DataBackup 1

### Timeer configuration--automatic backup of the storage time of user authentication data
* The unit is in minutes. The default is 1.
* Warning: If there is a lot of data in the database, when the service is restarted, if the time to reload the data into the memory exceeds the set backup time, the data entity file will be overwritten, which is a very serious data loss error.  
>Task_DataAuthBackup 1

### Timer configuration--the time to automatically clean up expired data in the database
* The unit is in minutes. The default is 1.
>Task_ClearExpireData 1

### Timer configuration--time to automatically back up server cluster information data
* The unit is in minutes. The default is 1.
* Warning: If there is a lot of data in the database, when the service is restarted, if the time to reload the data into the memory exceeds the set backup time, the data entity file will be overwritten, which is a very serious data loss error.  
>Task_ServerListBackup 1

### Timer configuration--time to automatically back up Gossip service cluster information
* The unit is in minutes. The default is 1.
* Warning: If there is a lot of data in the database, when the service is restarted, if the time to reload the data into the memory exceeds the set backup time, the data entity file will be overwritten, which is a very serious data loss error.
>Task_ServerGossipListBackup 1

### Timer configuration--automatically output Owl memory information
* The unit is in minutes. The default is 1. 
>Task_MemoryInfoToLog 1

### Enable Task execution success log output. A value of 1 prints logs, and any other value masks logs.  
>Task_OutputSuccessLog 1